#!/usr/bin/env python
import argparse
import re


def get_old_version():
    with open("pyproject.toml") as fp:
        content = fp.read()
    match = re.search(r'^version = "(\d+\.\d+\.\d+)"$', content, flags=re.MULTILINE)
    assert match
    return match.group(1)


def replace_version(filename, formatstr, old_version, new_version):
    print(f"updating {filename}")
    with open(filename) as fp:
        content = fp.read()
    old_str = formatstr.format(old_version)
    new_str = formatstr.format(new_version)
    content = content.replace(old_str, new_str)
    with open(filename, "w") as fp:
        fp.write(content)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-n", "--number", help="dev number to use, defaults to 1", type=int, default=1
    )
    args = parser.parse_args()

    old_version = get_old_version()
    new_version = old_version + f".dev{args.number}"

    replace_version("pyproject.toml", 'version = "{}"', old_version, new_version)
    print("done")


if __name__ == "__main__":
    main()



import json

import pytest

CASE1_MAIN_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "properties": {
        "title": {"$ref": "./title_schema.json"},
    },
    "additionalProperties": False,
}
CASE1_TITLE_SCHEMA = {
    "type": "string",
}
CASE1_PASSING_DOCUMENT = {"title": "doc one"}
CASE1_FAILING_DOCUMENT = {"title": 2}


CASE2_MAIN_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "type": "object",
    "required": ["test"],
    "properties": {"test": {"$ref": "./values.json#/$defs/test"}},
}
CASE2_VALUES_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "$defs": {"test": {"type": "string"}},
}
CASE2_PASSING_DOCUMENT = {"test": "some data"}
CASE2_FAILING_DOCUMENT = {"test": {"foo": "bar"}}


def _prep_files(tmp_path, main_schema, other_schema_data, instance):
    main_schemafile = tmp_path / "main_schema.json"
    main_schemafile.write_text(json.dumps(main_schema))
    for k, v in other_schema_data.items():
        schemafile = tmp_path / k
        schemafile.write_text(json.dumps(v))
    doc = tmp_path / "doc.json"
    doc.write_text(json.dumps(instance))
    return main_schemafile, doc


@pytest.mark.parametrize(
    "main_schema, other_schema_data, instance",
    [
        (
            CASE1_MAIN_SCHEMA,
            {"title_schema.json": CASE1_TITLE_SCHEMA},
            CASE1_PASSING_DOCUMENT,
        ),
        (
            CASE2_MAIN_SCHEMA,
            {"values.json": CASE2_VALUES_SCHEMA},
            CASE2_PASSING_DOCUMENT,
        ),
    ],
)
@pytest.mark.parametrize("with_file_scheme", [True, False])
def test_local_ref_schema(
    run_line_simple,
    tmp_path,
    main_schema,
    other_schema_data,
    instance,
    with_file_scheme,
):
    main_schemafile, doc = _prep_files(
        tmp_path, main_schema, other_schema_data, instance
    )
    if with_file_scheme:
        schemafile = main_schemafile.resolve().as_uri()
    else:
        schemafile = str(main_schemafile)
    run_line_simple(["--schemafile", schemafile, str(doc)])


@pytest.mark.parametrize(
    "main_schema, other_schema_data, instance, expect_err",
    [
        (
            CASE1_MAIN_SCHEMA,
            {"title_schema.json": CASE1_TITLE_SCHEMA},
            CASE1_FAILING_DOCUMENT,
            None,
        ),
        (
            CASE2_MAIN_SCHEMA,
            {"values.json": CASE2_VALUES_SCHEMA},
            CASE2_FAILING_DOCUMENT,
            "{'foo': 'bar'} is not of type 'string'",
        ),
    ],
)
@pytest.mark.parametrize("with_file_scheme", [True, False])
def test_local_ref_schema_failure_case(
    run_line,
    tmp_path,
    main_schema,
    other_schema_data,
    instance,
    expect_err,
    with_file_scheme,
):
    main_schemafile, doc = _prep_files(
        tmp_path, main_schema, other_schema_data, instance
    )
    if with_file_scheme:
        schemafile = main_schemafile.resolve().as_uri()
    else:
        schemafile = str(main_schemafile)
    res = run_line(["check-jsonschema", "--schemafile", schemafile, str(doc)])
    assert res.exit_code == 1
    if expect_err is not None:
        assert expect_err in res.stdout



"""
Identify filetypes by extension
"""

from __future__ import annotations

import pathlib

_EXTENSION_MAP = {
    "cff": "yaml",
    "json": "json",
    "jsonld": "json",
    "geojson": "json",
    "yaml": "yaml",
    "yml": "yaml",
    "ymlld": "yaml",
    "eyaml": "yaml",
    "json5": "json5",
    "toml": "toml",
}


def path_to_type(path: str | pathlib.Path, *, default_type: str = "json") -> str:
    if isinstance(path, str):
        ext = path.rpartition(".")[2]
    else:
        ext = path.suffix.lstrip(".")

    if ext in _EXTENSION_MAP:
        return _EXTENSION_MAP[ext]

    return default_type



import click
import pytest

from check_jsonschema.cli.warnings import deprecation_warning_callback


@click.command("foo")
@click.option(
    "--bar",
    is_flag=True,
    callback=deprecation_warning_callback("--bar", is_flag=True),
)
@click.option(
    "--baz",
    callback=deprecation_warning_callback(
        "--baz", append_message="Use --frob instead!"
    ),
)
def mycli(bar, baz):
    print(bar)
    if baz:
        print(baz)


def test_deprecation_warning_callback_on_missing_opts(cli_runner):
    result = cli_runner.invoke(mycli, [])
    assert result.exit_code == 0
    assert result.stdout == "False\n"


def test_deprecation_warning_callback_on_flag(cli_runner):
    with pytest.warns(
        UserWarning,
        match="'--bar' is deprecated and will be removed in a future release",
    ):
        result = cli_runner.invoke(mycli, ["--bar"], catch_exceptions=False)
    assert result.exit_code == 0, result.stdout
    assert result.stdout == "True\n"


def test_deprecation_warning_callback_added_message(cli_runner):
    with pytest.warns(
        UserWarning,
        match=(
            "'--baz' is deprecated and will be removed in a future release. "
            "Use --frob instead!"
        ),
    ):
        result = cli_runner.invoke(mycli, ["--baz", "ok"], catch_exceptions=False)
    assert result.exit_code == 0, result.stdout
    assert result.stdout == "False\nok\n"



import inspect
import os
import pathlib
import sys

import pytest
import responses
from click.testing import CliRunner


@pytest.fixture
def cli_runner():
    # compatibility for click==8.2.0 vs click<=8.1
    sig = inspect.signature(CliRunner)
    if "mix_stderr" in sig.parameters:
        return CliRunner(mix_stderr=False)
    return CliRunner()


@pytest.fixture(autouse=True)
def mocked_responses():
    responses.start()
    yield
    responses.stop()
    responses.reset()


@pytest.fixture
def mock_module(tmp_path, monkeypatch):
    monkeypatch.syspath_prepend(tmp_path)
    all_names_to_clear = []

    def func(path, text):
        path = pathlib.Path(path)
        mod_dir = tmp_path / path.parent
        mod_dir.mkdir(parents=True, exist_ok=True)
        for part in path.parts[:-1]:
            (tmp_path / part / "__init__.py").touch()

        (tmp_path / path).write_text(text)

        for i in range(len(path.parts)):
            modname = ".".join(path.parts[: i + 1])
            if modname.endswith(".py"):
                modname = modname[:-3]
            all_names_to_clear.append(modname)

    yield func

    for name in all_names_to_clear:
        if name in sys.modules:
            del sys.modules[name]


@pytest.fixture
def in_tmp_dir(request, tmp_path):
    os.chdir(str(tmp_path))
    yield
    os.chdir(request.config.invocation_dir)


@pytest.fixture
def cache_dir(tmp_path):
    return tmp_path / ".cache"


@pytest.fixture(autouse=True)
def patch_cache_dir(monkeypatch, cache_dir):
    with monkeypatch.context() as m:
        m.setattr(
            "check_jsonschema.cachedownloader._base_cache_dir", lambda: str(cache_dir)
        )
        yield m


@pytest.fixture
def url2cachepath():
    from check_jsonschema.cachedownloader import url_to_cache_filename

    def _get(cache_dir, url):
        return cache_dir / url_to_cache_filename(url)

    return _get


@pytest.fixture
def downloads_cache_dir(tmp_path):
    return tmp_path / ".cache" / "check_jsonschema" / "downloads"


@pytest.fixture
def get_download_cache_loc(downloads_cache_dir, url2cachepath):
    def _get(url):
        return url2cachepath(downloads_cache_dir, url)

    return _get


@pytest.fixture
def inject_cached_download(downloads_cache_dir, get_download_cache_loc):
    def _write(uri, content):
        downloads_cache_dir.mkdir(parents=True)
        path = get_download_cache_loc(uri)
        if isinstance(content, str):
            path.write_text(content)
        else:
            path.write_bytes(content)

    return _write


@pytest.fixture
def refs_cache_dir(tmp_path):
    return tmp_path / ".cache" / "check_jsonschema" / "refs"


@pytest.fixture
def get_ref_cache_loc(refs_cache_dir, url2cachepath):
    def _get(url):
        return url2cachepath(refs_cache_dir, url)

    return _get


@pytest.fixture
def inject_cached_ref(refs_cache_dir, get_ref_cache_loc):
    def _write(uri, content):
        refs_cache_dir.mkdir(parents=True)
        get_ref_cache_loc(uri).write_text(content)

    return _write



#!/usr/bin/env python3
from __future__ import annotations

import importlib.metadata
import typing as t

from check_jsonschema.catalog import SCHEMA_CATALOG

version = importlib.metadata.version("check_jsonschema")


def iter_catalog_hooks():
    for name in SCHEMA_CATALOG:
        # copy config (new dict)
        config = dict(SCHEMA_CATALOG[name]["hook_config"])
        # set computed attributes
        config["schema_name"] = name
        config["id"] = f"check-{name}"
        config["description"] = (
            config.get("description")
            or f"{config['name']} against the schema provided by SchemaStore"
        )
        if "types" in config and isinstance(config["types"], str):
            config["types"] = [config["types"]]
        yield config


def update_hook_config(new_config: str) -> None:
    print("updating .pre-commit-hooks.yaml")
    with open(".pre-commit-hooks.yaml") as fp:
        content = fp.read()

    autogen_marker = "# --AUTOGEN_HOOKS_START-- #"

    # get the first part of the content (and assert that it's OK)
    content = content.split(autogen_marker)[0]
    assert "check-jsonschema" in content

    content = content + autogen_marker + "\n\n" + new_config
    with open(".pre-commit-hooks.yaml", "w") as fp:
        fp.write(content)


def generate_hook_lines(config) -> t.Iterator[str]:
    yield "# this hook is autogenerated from a script"
    yield "# to modify this hook, update `src/check_jsonschema/catalog.py`"
    yield "# and run `just generate-hooks` or `tox run -e generate-hooks-config`"
    yield f"- id: {config['id']}"
    yield f"  name: {config['name']}"
    yield f"  description: '{config['description']}'"

    add_args = " ".join(config.get("add_args", []))
    if add_args:
        add_args = " " + add_args
    yield (
        "  entry: check-jsonschema --builtin-schema "
        f"vendor.{config['schema_name']}{add_args}"
    )

    yield "  language: python"

    if isinstance(config["files"], list):
        config["files"] = r""">
    (?x)^(
      {}
    )$""".format("|\n      ".join(config["files"]))

    yield f"  files: {config['files']}"

    if "types" in config:
        yield f"  types: [{','.join(config['types'])}]"
    if "types_or" in config:
        yield f"  types_or: [{','.join(config['types_or'])}]"

    yield ""


def generate_all_hook_config_lines() -> t.Iterator[str]:
    for hook_config in iter_catalog_hooks():
        yield from generate_hook_lines(hook_config)


def format_all_hook_config() -> str:
    return "\n".join(generate_all_hook_config_lines())


def update_usage_list_schemas() -> None:
    print("updating docs/usage.rst -- list schemas")
    with open("docs/usage.rst") as fp:
        content = fp.read()

    vendored_list_start = ".. vendored-schema-list-start\n"
    vendored_list_end = "\n.. vendored-schema-list-end"

    content_head = content.split(vendored_list_start)[0]
    content_tail = content.split(vendored_list_end)[-1]

    generated_list = "\n".join(
        [vendored_list_start]
        + [f"- ``vendor.{n}``" for n in SCHEMA_CATALOG]
        + [vendored_list_end]
    )

    content = content_head + generated_list + content_tail
    with open("docs/usage.rst", "w") as fp:
        fp.write(content)


def update_precommit_usage_supported_hooks() -> None:
    print("updating docs/precommit_usage.rst -- generated hooks")
    with open("docs/precommit_usage.rst") as fp:
        content = fp.read()

    generated_list_start = ".. generated-hook-list-start\n"
    generated_list_end = ".. generated-hook-list-end"

    content_head = content.split(generated_list_start)[0]
    content_tail = content.split(generated_list_end)[-1]

    generated_list = "\n\n".join([generated_list_start] + [f"""\
``{config["id"]}``
{"~" * (len(config["id"]) + 4)}

{config["description"]}

.. code-block:: yaml
    :caption: example config

    - repo: https://github.com/python-jsonschema/check-jsonschema
      rev: {version}
      hooks:
        - id: {config["id"]}
""" for config in iter_catalog_hooks()] + [generated_list_end])

    content = content_head + generated_list + content_tail
    with open("docs/precommit_usage.rst", "w") as fp:
        fp.write(content)


def update_docs() -> None:
    update_usage_list_schemas()
    update_precommit_usage_supported_hooks()


def main() -> None:
    update_hook_config(format_all_hook_config())
    update_docs()


if __name__ == "__main__":
    main()



import multiprocessing
import os
import platform
import sys

import pytest
import responses


@pytest.mark.skipif(
    platform.system() != "Linux", reason="test requires /proc/self/ mechanism"
)
@pytest.mark.skipif(sys.version_info < (3, 8), reason="test uses os.memfd_create")
def test_schema_and_instance_in_memfds(run_line_simple):
    """
    create memory file descriptors and write schema and instance data into those
    ensure the result works when the paths to those fds are passed on the CLI
    """
    schemafd = os.memfd_create("test_memfd_schema")
    instancefd = os.memfd_create("test_memfd_instance")
    try:
        os.write(schemafd, b'{"type": "integer"}')
        os.write(instancefd, b"42")

        schema_path = f"/proc/self/fd/{schemafd}"
        instance_path = f"/proc/self/fd/{instancefd}"

        run_line_simple(["--schemafile", schema_path, instance_path])
    finally:
        os.close(schemafd)
        os.close(instancefd)


# helper (in global scope) for multiprocessing "spawn" to be able to use to launch
# background writers
def _fifo_write(path, data):
    fd = os.open(path, os.O_WRONLY)
    try:
        os.write(fd, data)
    finally:
        os.close(fd)


@pytest.mark.skipif(os.name != "posix", reason="test requires mkfifo")
@pytest.mark.parametrize("check_succeeds", (True, False))
def test_schema_and_instance_in_fifos(tmp_path, run_line, check_succeeds):
    """
    create fifos and write schema and instance data into those
    ensure the result works when the paths to those fds are passed on the CLI
    """
    schema_path = tmp_path / "schema"
    instance_path = tmp_path / "instance"
    os.mkfifo(schema_path)
    os.mkfifo(instance_path)

    spawn_ctx = multiprocessing.get_context("spawn")

    schema_proc = spawn_ctx.Process(
        target=_fifo_write, args=(schema_path, b'{"type": "integer"}')
    )
    schema_proc.start()
    instance_data = b"42" if check_succeeds else b'"foo"'
    instance_proc = spawn_ctx.Process(
        target=_fifo_write, args=(instance_path, instance_data)
    )
    instance_proc.start()

    try:
        result = run_line(
            ["check-jsonschema", "--schemafile", str(schema_path), str(instance_path)]
        )
        if check_succeeds:
            assert result.exit_code == 0
        else:
            assert result.exit_code == 1
    finally:
        schema_proc.terminate()
        instance_proc.terminate()


@pytest.mark.parametrize("check_passes", (True, False))
def test_remote_schema_requiring_retry(run_line, check_passes, tmp_path):
    """
    a "remote schema" (meaning HTTPS) with bad data, therefore requiring that a retry
    fires in order to parse
    """
    schema_loc = "https://example.com/schema1.json"
    responses.add("GET", schema_loc, body="", match_querystring=None)
    responses.add(
        "GET",
        schema_loc,
        headers={"Last-Modified": "Sun, 01 Jan 2000 00:00:01 GMT"},
        json={"type": "integer"},
        match_querystring=None,
    )

    instance_path = tmp_path / "instance.json"
    instance_path.write_text("42" if check_passes else '"foo"')

    result = run_line(
        ["check-jsonschema", "--schemafile", schema_loc, str(instance_path)]
    )
    if check_passes:
        assert result.exit_code == 0
    else:
        assert result.exit_code == 1


@pytest.mark.parametrize("check_passes", (True, False))
@pytest.mark.parametrize("using_stdin", ("schema", "instance"))
def test_schema_or_instance_from_stdin(
    run_line, check_passes, tmp_path, monkeypatch, using_stdin
):
    """
    a "remote schema" (meaning HTTPS) with bad data, therefore requiring that a retry
    fires in order to parse
    """
    if using_stdin == "schema":
        instance_path = tmp_path / "instance.json"
        instance_path.write_text("42" if check_passes else '"foo"')

        result = run_line(
            ["check-jsonschema", "--schemafile", "-", str(instance_path)],
            input='{"type": "integer"}',
        )
    elif using_stdin == "instance":
        schema_path = tmp_path / "schema.json"
        schema_path.write_text('{"type": "integer"}')
        instance = "42" if check_passes else '"foo"'

        result = run_line(
            ["check-jsonschema", "--schemafile", schema_path, "-"],
            input=instance,
        )
    else:
        raise NotImplementedError
    if check_passes:
        assert result.exit_code == 0
    else:
        assert result.exit_code == 1



import argparse
import sys
from collections import defaultdict
from xml.etree import ElementTree  # nosec


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("FILES", nargs="+")
    args = parser.parse_args()

    tests_by_name = defaultdict(dict)
    skipped_module_counts = defaultdict(int)
    for filename in args.FILES:
        tree = ElementTree.parse(filename)
        root = tree.getroot()

        for testcase in root.findall("./testsuite/testcase"):
            classname = testcase.get("classname")
            name = testcase.get("name")

            skip_node = testcase.find("skipped")

            if classname:
                nodename = f"{classname.replace('.', '/')}.py::{name}"
                if skip_node is not None:
                    if "skipped" not in tests_by_name[nodename]:
                        tests_by_name[nodename]["skipped"] = True
                else:
                    tests_by_name[nodename]["skipped"] = False
            else:
                if skip_node is not None:
                    skipped_module_counts[name] += 1

    skipped_modules = {
        modname
        for modname, count in skipped_module_counts.items()
        if count == len(args.FILES)
    }

    fail = False
    for nodename, attributes in tests_by_name.items():
        if attributes.get("skipped") is True:
            print(f"ALWAYS SKIPPED: {nodename}")
            fail = True

    if skipped_modules:
        for modname in skipped_modules:
            print(f"ALWAYS SKIPPED MODULE: {modname}")
        fail = True

    if fail:
        print("fail")
        sys.exit(1)
    print("ok")
    sys.exit(0)


if __name__ == "__main__":
    main()



import json

import pytest
import responses

from check_jsonschema.parsers.json5 import ENABLED as JSON5_ENABLED

SIMPLE_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "properties": {
        "title": {"type": "string"},
    },
    "additionalProperties": False,
}
YAML_REF_MAIN_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "properties": {
        "title": {"$ref": "./title_schema.yaml"},
    },
    "additionalProperties": False,
}
JSON5_REF_MAIN_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "properties": {
        "title": {"$ref": "./title_schema.json5"},
    },
    "additionalProperties": False,
}
TITLE_SCHEMA = {"type": "string"}
PASSING_DOCUMENT = {"title": "doc one"}
FAILING_DOCUMENT = {"title": 2}


@pytest.mark.parametrize("passing_data", [True, False])
def test_yaml_reference(run_line, tmp_path, passing_data):
    main_schemafile = tmp_path / "main_schema.json"
    main_schemafile.write_text(json.dumps(YAML_REF_MAIN_SCHEMA))
    # JSON is a subset of YAML, so this works for generated YAML
    ref_schema = tmp_path / "title_schema.yaml"
    ref_schema.write_text(json.dumps(TITLE_SCHEMA))

    doc = tmp_path / "doc.json"
    if passing_data:
        doc.write_text(json.dumps(PASSING_DOCUMENT))
    else:
        doc.write_text(json.dumps(FAILING_DOCUMENT))

    result = run_line(
        ["check-jsonschema", "--schemafile", str(main_schemafile), str(doc)]
    )
    assert result.exit_code == (0 if passing_data else 1)


@pytest.mark.skipif(not JSON5_ENABLED, reason="test requires json5")
@pytest.mark.parametrize("passing_data", [True, False])
def test_json5_reference(run_line, tmp_path, passing_data):
    main_schemafile = tmp_path / "main_schema.json"
    main_schemafile.write_text(json.dumps(JSON5_REF_MAIN_SCHEMA))
    ref_schema = tmp_path / "title_schema.json5"
    ref_schema.write_text(json.dumps(TITLE_SCHEMA))

    doc = tmp_path / "doc.json"
    if passing_data:
        doc.write_text(json.dumps(PASSING_DOCUMENT))
    else:
        doc.write_text(json.dumps(FAILING_DOCUMENT))

    result = run_line(
        ["check-jsonschema", "--schemafile", str(main_schemafile), str(doc)]
    )
    assert result.exit_code == (0 if passing_data else 1)


@pytest.mark.skipif(not JSON5_ENABLED, reason="test requires json5")
@pytest.mark.parametrize("passing_data", [True, False])
def test_can_load_json5_schema(run_line, tmp_path, passing_data):
    # dump JSON to the JSON5 file, this is fine
    main_schemafile = tmp_path / "main_schema.json5"
    main_schemafile.write_text(json.dumps(SIMPLE_SCHEMA))

    doc = tmp_path / "doc.json"
    if passing_data:
        doc.write_text(json.dumps(PASSING_DOCUMENT))
    else:
        doc.write_text(json.dumps(FAILING_DOCUMENT))

    result = run_line(
        ["check-jsonschema", "--schemafile", str(main_schemafile), str(doc)]
    )
    assert result.exit_code == (0 if passing_data else 1)


@pytest.mark.parametrize("passing_data", [True, False])
def test_can_load_remote_yaml_schema(run_line, tmp_path, passing_data):
    retrieval_uri = "https://example.org/retrieval/schemas/main.yaml"
    responses.add(
        "GET",
        retrieval_uri,
        body="""\
"$schema": "http://json-schema.org/draft-07/schema"
properties:
  title: {"type": "string"}
additionalProperties: false
""",
    )

    doc = tmp_path / "doc.json"
    doc.write_text(json.dumps(PASSING_DOCUMENT if passing_data else FAILING_DOCUMENT))

    result = run_line(["check-jsonschema", "--schemafile", retrieval_uri, str(doc)])
    assert result.exit_code == (0 if passing_data else 1)


@pytest.mark.parametrize("passing_data", [True, False])
def test_can_load_remote_yaml_schema_ref(run_line, tmp_path, passing_data):
    retrieval_uri = "https://example.org/retrieval/schemas/main.yaml"
    responses.add(
        "GET",
        retrieval_uri,
        body="""\
"$schema": "http://json-schema.org/draft-07/schema"
properties:
  "title": {"$ref": "./title_schema.yaml"}
additionalProperties: false
""",
    )
    responses.add(
        "GET",
        "https://example.org/retrieval/schemas/title_schema.yaml",
        body="type: string",
    )

    doc = tmp_path / "doc.json"
    doc.write_text(json.dumps(PASSING_DOCUMENT if passing_data else FAILING_DOCUMENT))

    result = run_line(["check-jsonschema", "--schemafile", retrieval_uri, str(doc)])
    assert result.exit_code == (0 if passing_data else 1)


def test_can_load_remote_yaml_schema_ref_from_cache(
    run_line, inject_cached_ref, tmp_path
):
    retrieval_uri = "https://example.org/retrieval/schemas/main.yaml"
    responses.add(
        "GET",
        retrieval_uri,
        body="""\
"$schema": "http://json-schema.org/draft-07/schema"
properties:
  "title": {"$ref": "./title_schema.yaml"}
additionalProperties: false
""",
    )

    ref_loc = "https://example.org/retrieval/schemas/title_schema.yaml"
    # populate a bad schema, but then "override" that with a good cache value
    # this can only pass (in the success case) if the cache loading really works
    responses.add("GET", ref_loc, body="false")
    inject_cached_ref(ref_loc, "type: string")

    doc = tmp_path / "doc.json"
    doc.write_text(json.dumps(PASSING_DOCUMENT))

    result = run_line(["check-jsonschema", "--schemafile", retrieval_uri, str(doc)])
    assert result.exit_code == 0



import json
import textwrap

import pytest
from jsonschema import Draft7Validator

from check_jsonschema.parsers import ParseError
from check_jsonschema.reporter import JsonReporter, TextReporter
from check_jsonschema.result import CheckResult


def _make_success_result():
    res = CheckResult()
    res.successes.append("foo.json")
    return res


@pytest.mark.parametrize("verbosity", (0, 1, 2, 3))
@pytest.mark.parametrize("use_report_result_path", (False, True))
def test_text_format_success(capsys, verbosity, use_report_result_path):
    reporter = TextReporter(verbosity=verbosity)
    if use_report_result_path:
        reporter.report_result(_make_success_result())
    else:
        reporter.report_success(_make_success_result())
    captured = capsys.readouterr()
    assert captured.err == ""
    if verbosity == 0:
        assert captured.out == ""
    elif verbosity < 3:
        assert captured.out == "ok -- validation done\n"
    else:
        assert captured.out == textwrap.dedent("""\
            ok -- validation done
            The following files were checked:
              foo.json
            """)


@pytest.mark.parametrize("verbosity", (0, 1, 2))
@pytest.mark.parametrize("use_report_result_path", (False, True))
def test_json_format_success(capsys, verbosity, use_report_result_path):
    reporter = JsonReporter(verbosity=verbosity, pretty=False)
    if use_report_result_path:
        reporter.report_result(_make_success_result())
    else:
        reporter.report_success(_make_success_result())
    captured = capsys.readouterr()
    assert captured.err == ""
    if verbosity == 0:
        assert captured.out == '{"status":"ok"}\n'
    elif verbosity == 1:
        assert captured.out == '{"status":"ok","errors":[]}\n'
    else:
        assert (
            captured.out == '{"status":"ok","errors":[],"checked_paths":["foo.json"]}\n'
        )


def test_text_format_validation_error_message_simple():
    validator = Draft7Validator(
        {
            "properties": {
                "foo": {
                    "anyOf": [
                        {"type": "string"},
                        {"properties": {"bar": {"type": "array"}}},
                    ],
                },
            },
        },
    )
    err = next(validator.iter_errors({"foo": {"bar": 1}}))

    text_reporter = TextReporter(verbosity=1)
    s1 = text_reporter._format_validation_error_message(err, filename="foo.json")
    assert s1 == (
        "\x1b[33mfoo.json::$.foo\x1b[0m: {'bar': 1} "
        "is not valid under any of the given schemas"
    )

    s2 = text_reporter._format_validation_error_message(err)
    assert s2 == (
        "\x1b[33m$.foo\x1b[0m: {'bar': 1} "
        "is not valid under any of the given schemas"
    )


@pytest.mark.parametrize("verbosity", (0, 1, 2))
def test_text_print_validation_error_nested(capsys, verbosity):
    validator = Draft7Validator(
        {
            "anyOf": [
                {
                    "properties": {
                        "foo": {
                            "oneOf": [
                                {"type": "string"},
                                {"type": "integer"},
                            ],
                        },
                    },
                },
                {
                    "properties": {
                        "bar": {
                            "oneOf": [
                                {"type": "string"},
                                {
                                    "type": "object",
                                    "properties": {"baz": {"type": "integer"}},
                                },
                            ]
                        }
                    }
                },
            ]
        },
    )
    err = next(validator.iter_errors({"foo": {}, "bar": {"baz": "buzz"}}))

    result = CheckResult()
    result.record_validation_error("foo.json", err)

    text_reporter = TextReporter(verbosity=verbosity)
    text_reporter.report_result(result)
    captured = capsys.readouterr()
    # nothing to stderr
    assert captured.err == ""

    # if verbosity<1 stop here
    if verbosity < 1:
        assert captured.out == ""
        return

    # only assert part of the message
    # dict member order isn't guaranteed and isn't relevant here
    assert "is not valid under any of the given schemas" in captured.out
    assert "Underlying errors" in captured.out

    # we don't know which error was the best match (algo for best match could change)
    # so just assert the presence of the underlying error messages at higher verbosity
    if verbosity > 1:
        assert "$.foo: {} is not of type 'string'" in captured.out
        assert "$.bar: {'baz': 'buzz'} is not of type 'string'" in captured.out
        assert "$.bar.baz: 'buzz' is not of type 'integer'" in captured.out
    else:
        assert (
            "4 other errors were produced. Use '--verbose' to see all errors."
            in captured.out
        )


@pytest.mark.parametrize("pretty_json", (True, False))
@pytest.mark.parametrize("verbosity", (0, 1, 2))
def test_json_format_validation_error_nested(capsys, pretty_json, verbosity):
    validator = Draft7Validator(
        {
            "anyOf": [
                {
                    "properties": {
                        "foo": {
                            "oneOf": [
                                {"type": "string"},
                                {"type": "integer"},
                            ],
                        },
                    },
                },
                {
                    "properties": {
                        "bar": {
                            "oneOf": [
                                {"type": "string"},
                                {
                                    "type": "object",
                                    "properties": {"baz": {"type": "integer"}},
                                },
                            ]
                        }
                    }
                },
            ]
        },
    )
    err = next(validator.iter_errors({"foo": {}, "bar": {"baz": "buzz"}}))

    result = CheckResult()
    result.record_validation_error("foo.json", err)

    json_reporter = JsonReporter(pretty=pretty_json, verbosity=verbosity)
    json_reporter.report_result(result)
    captured = capsys.readouterr()
    # nothing to stderr
    assert captured.err == ""

    # json data to stdout, parse it
    data = json.loads(captured.out)
    assert data["status"] == "fail"

    # stop here unless verbosity>=1
    if verbosity < 1:
        assert data == {"status": "fail"}
        return

    assert len(data["errors"]) == 1
    assert "is not valid under any of the given schemas" in data["errors"][0]["message"]
    assert data["errors"][0]["has_sub_errors"]
    assert data["errors"][0]["num_sub_errors"] == 5

    # stop here unless 'verbosity>=2'
    if verbosity < 2:
        assert "sub_errors" not in data["errors"][0]
        return
    else:
        assert "sub_errors" in data["errors"][0]

    sub_errors = data["errors"][0]["sub_errors"]

    foo_errors, bar_errors, bar_baz_errors = [], [], []
    for error_item in sub_errors:
        if error_item["path"] == "$.foo":
            foo_errors.append(error_item)
        elif error_item["path"] == "$.bar":
            bar_errors.append(error_item)
        elif error_item["path"] == "$.bar.baz":
            bar_baz_errors.append(error_item)
    assert len(foo_errors) == 3
    assert len(bar_baz_errors) == 1
    assert len(bar_errors) == 2

    assert "'buzz' is not of type 'integer'" == bar_baz_errors[0]["message"]
    assert {item["message"] for item in foo_errors} == {
        "{} is not of type 'string'",
        "{} is not of type 'integer'",
        "{} is not valid under any of the given schemas",
    }

    assert "{'baz': 'buzz'} is not of type 'string'" in [
        item["message"] for item in bar_errors
    ]


def test_text_print_parse_error_with_cause(capsys):
    cause = json.JSONDecodeError("a bad thing happened", "{,}", 1)
    error = ParseError("whoopsie during parsing")
    error.__cause__ = cause

    result = CheckResult()
    result.record_parse_error("foo.json", error)

    text_reporter = TextReporter(verbosity=1)
    text_reporter.report_result(result)
    captured = capsys.readouterr()

    # nothing to stderr
    assert captured.err == ""
    # stdout contains a nicely formatted error
    assert textwrap.dedent("""\
            Several files failed to parse.
              whoopsie during parsing
                JSONDecodeError: a bad thing happened: line 1 column 2 (char 1)
            """) in captured.out



from .errors import SchemaParseError, UnsupportedUrlScheme
from .main import BuiltinSchemaLoader, MetaSchemaLoader, SchemaLoader, SchemaLoaderBase

__all__ = (
    "SchemaParseError",
    "UnsupportedUrlScheme",
    "BuiltinSchemaLoader",
    "MetaSchemaLoader",
    "SchemaLoader",
    "SchemaLoaderBase",
)



from __future__ import annotations

import linecache
import textwrap
import traceback
import typing as t

import click


def format_error_message(err: BaseException) -> str:
    return f"{type(err).__name__}: {err}"


def format_minimal_error(err: BaseException, *, indent: int = 0) -> str:
    lines = [textwrap.indent(str(err), indent * " ")]
    if err.__cause__ is not None:
        lines.append(
            textwrap.indent(format_error_message(err.__cause__), (indent + 2) * " ")
        )

    return "\n".join(lines)


def format_shortened_error(err: BaseException, *, indent: int = 0) -> str:
    lines = []
    lines.append(textwrap.indent(format_error_message(err), indent * " "))
    if err.__traceback__ is not None:
        lineno = err.__traceback__.tb_lineno
        tb_frame = err.__traceback__.tb_frame
        filename = tb_frame.f_code.co_filename
        line = linecache.getline(filename, lineno)
        lines.append((indent + 2) * " " + f'in "{filename}", line {lineno}')
        lines.append((indent + 2) * " " + ">>> " + line.strip())
    return "\n".join(lines)


def format_shortened_trace(caught_err: BaseException) -> str:
    err_stack: list[BaseException] = [caught_err]
    while err_stack[-1].__context__ is not None:
        err_stack.append(err_stack[-1].__context__)  # type: ignore[arg-type]

    parts = [format_shortened_error(caught_err)]
    indent = 0
    for err in err_stack[1:]:
        indent += 2
        parts.append("\n" + indent * " " + "caused by\n")
        parts.append(format_shortened_error(err, indent=indent))
    return "\n".join(parts)


def format_error(
    err: Exception, mode: t.Literal["minimal", "short", "full"] = "short"
) -> str:
    if mode == "minimal":
        return format_minimal_error(err)
    elif mode == "short":
        return format_shortened_trace(err)
    else:
        return "".join(traceback.format_exception(type(err), err, err.__traceback__))


def print_error(
    err: Exception, mode: t.Literal["minimal", "short", "full"] = "short"
) -> None:
    click.echo(format_error(err, mode=mode), err=True)



#!/usr/bin/env python3
from __future__ import annotations

import datetime
import hashlib
import os
import re
import subprocess
import textwrap

import requests

from check_jsonschema.catalog import SCHEMA_CATALOG

TODAY = datetime.datetime.today().strftime("%Y-%m-%d")

VENDOR_SLUG = "\n.. vendor-insert-here\n"

EXISTING_CHANGELINE_PATTERN = re.compile(
    re.escape(VENDOR_SLUG)
    + "\n?- Update vendored schemas: "
    + r"([\w-]+(,\s+[\w-]+)*)\s+"
    + r"\(\d{4}-\d{2}-\d{2}\)"
    + "\n",
    flags=re.MULTILINE,
)

OLD_HASHES: dict[str, str] = {}
UPDATED_SCHEMAS: set[str] = set()
SCHEMAS_WITH_NEW_HASHES: set[str] = set()


def schema2filename(name: str) -> str:
    return f"src/check_jsonschema/builtin_schemas/vendor/{name}.json"


def schema2hashfile(name: str) -> str:
    return f"src/check_jsonschema/builtin_schemas/vendor/sha256/{name}.sha256"


def file2digest(filepath: str) -> str:
    sha = hashlib.sha256()
    with open(filepath, "rb") as fp:
        sha.update(fp.read())
    return sha.hexdigest()


def load_old_hashes() -> None:
    for name, _config in SCHEMA_CATALOG.items():
        hashfile = schema2hashfile(name)
        if os.path.exists(hashfile):
            with open(hashfile) as fp:
                OLD_HASHES[name] = fp.read().strip()


def normalize_schema_contents(data: bytes) -> bytes:
    lines = data.split(b"\n")
    return b"\n".join(line.rstrip(b" ") for line in lines)


def download_schemas() -> None:
    print("downloading schemas to check for updates")
    session = requests.Session()

    for schema_name, config in SCHEMA_CATALOG.items():
        schema_url = config["url"]

        print(f"  {schema_name} ({schema_url})")
        res = session.get(schema_url)
        new_content = normalize_schema_contents(res.content)

        sha = hashlib.sha256()
        sha.update(new_content)
        new_digest = sha.hexdigest()

        # early abort if downloaded content matches old hash
        if new_digest == OLD_HASHES.get(schema_name):
            continue

        with open(schema2filename(schema_name), "wb") as fp:
            fp.write(new_content)
        UPDATED_SCHEMAS.add(schema_name)


def run_precommit() -> None:
    if not UPDATED_SCHEMAS:
        print("no downloads updated, skipping pre-commit")
        return

    print("running pre-commit")
    cmd = ["pre-commit", "run", "--files"]
    for name in UPDATED_SCHEMAS:
        cmd.append(schema2filename(name))
    res = subprocess.run(cmd, capture_output=True)
    if res.returncode == 0:
        return


def save_new_hashes() -> None:
    print("saving new hashes")

    def _save_new_hash(name: str, digest: str) -> None:
        print(f"  - new hash for '{name}'")
        hashfile = schema2hashfile(name)
        with open(hashfile, "w", encoding="utf-8") as fp:
            fp.write(digest)
        SCHEMAS_WITH_NEW_HASHES.add(name)

    for name in UPDATED_SCHEMAS:
        digest = file2digest(schema2filename(name))

        # new file, changes were made and hash should update, OR
        # if the existing hash does not match the new hash, save the new one
        if name not in OLD_HASHES or digest != OLD_HASHES[name]:
            _save_new_hash(name, digest)


def check_changes_made() -> bool:
    return bool(SCHEMAS_WITH_NEW_HASHES)


def update_changelog() -> None:
    print("changes were made, updating changelog")
    with open("CHANGELOG.rst", encoding="utf-8") as fp:
        content = fp.read()

    new_item_prefix = "- Update vendored schemas:"

    if match := EXISTING_CHANGELINE_PATTERN.search(content):
        updated_schemas = {s.rstrip(",") for s in match.group(1).split()}
        updated_schemas.update(SCHEMAS_WITH_NEW_HASHES)
    else:
        updated_schemas = SCHEMAS_WITH_NEW_HASHES

    schema_list = sorted(updated_schemas)
    new_item = new_item_prefix + f" {', '.join(schema_list)} ({TODAY})\n"

    new_item = "\n".join(
        textwrap.wrap(
            new_item, break_on_hyphens=False, subsequent_indent="  ", width=88
        )
    )
    new_item = f"{VENDOR_SLUG}\n{new_item}\n"

    if match:
        content = EXISTING_CHANGELINE_PATTERN.sub(new_item, content)
    else:
        content = content.replace(VENDOR_SLUG, new_item)

    with open("CHANGELOG.rst", "w", encoding="utf-8") as fp:
        fp.write(content)


def main() -> None:
    print("vendor-schemas:BEGIN")
    load_old_hashes()
    download_schemas()
    run_precommit()
    save_new_hashes()
    if check_changes_made():
        update_changelog()
    else:
        print("no changes detected")
    print("vendor-schemas:END")


if __name__ == "__main__":
    main()



import os
import pathlib

import pytest
import responses

from check_jsonschema.schema_loader import SchemaLoader, SchemaParseError
from check_jsonschema.schema_loader.readers import HttpSchemaReader, LocalSchemaReader


@pytest.fixture
def in_tmp_dir(request, tmp_path):
    os.chdir(str(tmp_path))
    yield
    os.chdir(request.config.invocation_dir)


@pytest.mark.parametrize(
    "filename",
    [
        "schema.json",
        "schema.yaml",
    ],
)
def test_schemaloader_path_handling_relative_local_path(in_tmp_dir, filename):
    # ensure that the file exists so that the behavior of pathlib resolution will be
    # correct on Windows with older python versions
    # see: https://bugs.python.org/issue38671
    path = pathlib.Path("path", "to") / filename
    path.parent.mkdir(parents=True)
    path.touch()

    sl = SchemaLoader(str(path))
    assert isinstance(sl.reader, LocalSchemaReader)
    assert sl.reader.filename == os.path.abspath(str(path))
    assert str(sl.reader.path) == str(path.resolve())


@pytest.mark.parametrize(
    "filename",
    [
        "schema.yaml",
        "schema.yml",
        "https://foo.example.com/schema.yaml",
        "https://foo.example.com/schema.yml",
    ],
)
def test_schemaloader_yaml_data(tmp_path, filename):
    schema_text = """
---
"$schema": https://json-schema.org/draft/2020-12/schema
type: object
properties:
  a:
    type: object
    properties:
      b:
        type: array
        items:
          type: integer
      c:
        type: string
"""
    if filename.startswith("http"):
        responses.add("GET", filename, body=schema_text)
        path = filename
    else:
        f = tmp_path / filename
        f.write_text(schema_text)
        path = str(f)
    sl = SchemaLoader(path)
    schema = sl.get_schema()
    assert schema == {
        "$schema": "https://json-schema.org/draft/2020-12/schema",
        "type": "object",
        "properties": {
            "a": {
                "type": "object",
                "properties": {
                    "b": {"type": "array", "items": {"type": "integer"}},
                    "c": {"type": "string"},
                },
            },
        },
    }


@pytest.mark.parametrize(
    "schemafile",
    [
        "https://foo.example.com/schema.json",
        "http://foo.example.com/schema.json",
    ],
)
def test_schemaloader_remote_path(schemafile):
    sl = SchemaLoader(schemafile)
    assert isinstance(sl.reader, HttpSchemaReader)
    assert sl.reader.url == schemafile


def test_schemaloader_local_yaml_dup_anchor(tmp_path):
    f = tmp_path / "schema.yaml"
    f.write_text("""
---
"$schema": https://json-schema.org/draft/2020-12/schema
type: object
properties:
  a:
    type: object
    properties:
      b: &anchor
        type: array
        items:
          type: integer
      c: &anchor
        type: string
""")
    sl = SchemaLoader(str(f))
    schema = sl.get_schema()
    assert schema == {
        "$schema": "https://json-schema.org/draft/2020-12/schema",
        "type": "object",
        "properties": {
            "a": {
                "type": "object",
                "properties": {
                    "b": {"type": "array", "items": {"type": "integer"}},
                    "c": {"type": "string"},
                },
            },
        },
    }


def test_schemaloader_invalid_yaml_data(tmp_path):
    f = tmp_path / "foo.yaml"
    f.write_text("""\
a: {b
""")
    sl = SchemaLoader(str(f))
    with pytest.raises(SchemaParseError):
        sl.get_schema()



import json

SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema",
    "properties": {
        "title": {
            "type": "string",
            "default": "Untitled",
        },
    },
    "required": ["title"],
}

VALID_DOC = {
    "title": "doc one",
}

INVALID_DOC = {"title": {"foo": "bar"}}

MISSING_FIELD_DOC = {}


def test_run_with_fill_defaults_does_not_make_valid_doc_invalid(
    run_line_simple, tmp_path
):
    schemafile = tmp_path / "schema.json"
    schemafile.write_text(json.dumps(SCHEMA))

    doc = tmp_path / "instance.json"
    doc.write_text(json.dumps(VALID_DOC))

    run_line_simple(["--fill-defaults", "--schemafile", str(schemafile), str(doc)])


def test_run_with_fill_defaults_does_not_make_invalid_doc_valid(run_line, tmp_path):
    schemafile = tmp_path / "schema.json"
    schemafile.write_text(json.dumps(SCHEMA))

    doc = tmp_path / "instance.json"
    doc.write_text(json.dumps(INVALID_DOC))

    res = run_line(
        [
            "check-jsonschema",
            "--fill-defaults",
            "--schemafile",
            str(schemafile),
            str(doc),
        ]
    )
    assert res.exit_code == 1


def test_run_with_fill_defaults_adds_required_field(run_line, tmp_path):
    schemafile = tmp_path / "schema.json"
    schemafile.write_text(json.dumps(SCHEMA))

    doc = tmp_path / "instance.json"
    doc.write_text(json.dumps(MISSING_FIELD_DOC))

    # step 1: run without '--fill-defaults' and confirm failure
    result_without_fill_defaults = run_line(
        [
            "check-jsonschema",
            "--schemafile",
            str(schemafile),
            str(doc),
        ]
    )
    assert result_without_fill_defaults.exit_code == 1

    # step 2: run with '--fill-defaults' and confirm success
    result_with_fill_defaults = run_line(
        [
            "check-jsonschema",
            "--fill-defaults",
            "--schemafile",
            str(schemafile),
            str(doc),
        ]
    )
    assert result_with_fill_defaults.exit_code == 0



#!/usr/bin/env python
"""
Extract a changelog section from the full changelog contents, convert ReST and
sphinx-issues syntaxes to GitHub-flavored Markdown, and print the results.

Defaults to selecting the most recent (topmost) changelog section.
Can alternatively provide output for a specific version with `--target`.
e.g.

    ./scripts/changelog2md.py --target 3.20.0
"""

from __future__ import annotations

import argparse
import pathlib
import re
import typing as t

REPO_ROOT = pathlib.Path(__file__).parent.parent
CHANGELOG_PATH = REPO_ROOT / "CHANGELOG.rst"

CHANGELOG_HEADER_PATTERN = re.compile(r"^(\d+\.\d+\.\d+).*$", re.MULTILINE)

H2_RST_PATTERN = re.compile(r"-+")
H3_RST_PATTERN = re.compile(r"~+")

SPHINX_ISSUES_PR_PATTERN = re.compile(r":pr:`(\d+)`")
SPHINX_ISSUES_ISSUE_PATTERN = re.compile(r":issue:`(\d+)`")
SPHINX_ISSUES_USER_PATTERN = re.compile(r":user:`([^`]+)`")


def _trim_empty_lines(lines: list[str]) -> None:
    if not lines:
        return
    while lines[0] == "":
        lines.pop(0)
    while lines[-1] == "":
        lines.pop()


def _iter_target_section(target: str | None, changelog_content: str) -> t.Iterator[str]:
    started = False
    for line in changelog_content.split("\n"):
        if m := CHANGELOG_HEADER_PATTERN.match(line):
            if not started:
                if target is None or m.group(1) == target:
                    started = True
                continue
            else:
                return
        if H2_RST_PATTERN.fullmatch(line):
            continue
        if started:
            yield line


def get_last_changelog(changelog_content: str) -> list[str]:
    latest_changes = list(_iter_target_section(None, changelog_content))
    _trim_empty_lines(latest_changes)
    return latest_changes


def get_changelog_section(target: str, changelog_content: str) -> list[str]:
    lines = list(_iter_target_section(target, changelog_content))
    _trim_empty_lines(lines)
    return lines


def convert_rst_to_md(lines: list[str]) -> t.Iterator[str]:
    skip = False
    for i, line in enumerate(lines):
        if skip:
            skip = False
            continue

        try:
            peek = lines[i + 1]
        except IndexError:
            peek = None

        updated = line

        if peek is not None and H3_RST_PATTERN.fullmatch(peek):
            skip = True
            updated = f"## {updated}"

        updated = SPHINX_ISSUES_PR_PATTERN.sub(r"#\1", updated)
        updated = SPHINX_ISSUES_ISSUE_PATTERN.sub(r"#\1", updated)
        updated = SPHINX_ISSUES_USER_PATTERN.sub(r"@\1", updated)
        updated = updated.replace("``", "`")
        yield updated


def main():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument(
        "--target", "-t", help="A target version to use. Defaults to latest."
    )
    args = parser.parse_args()

    full_changelog = CHANGELOG_PATH.read_text()

    if args.target:
        changelog_section = get_changelog_section(args.target, full_changelog)
    else:
        changelog_section = get_last_changelog(full_changelog)

    for line in convert_rst_to_md(changelog_section):
        print(line)


if __name__ == "__main__":
    main()



import typing as t

import pytest

from check_jsonschema.cli import main as cli_main

click_type_test = pytest.importorskip(
    "click_type_test", reason="tests require 'click-type-test'"
)


def test_annotations_match_click_params():
    click_type_test.check_param_annotations(
        cli_main,
        overrides={
            # don't bother with a Literal for this, since it's relatively dynamic data
            "builtin_schema": str | None,
            # force default_filetype to be a Literal including `json5`, which is only
            # included in the choices if a parser is installed
            "default_filetype": t.Literal["json", "yaml", "toml", "json5"],
            "force_filetype": t.Literal["json", "yaml", "toml", "json5"] | None,
        },
    )



import json
import os
import platform
import time

import pytest
import requests
import responses

from check_jsonschema.cachedownloader import (
    CacheDownloader,
    FailedDownloadError,
    _cache_hit,
    _lastmod_from_response,
    url_to_cache_filename,
)

DEFAULT_RESPONSE_URL = "https://example.com/schema1.json"
DEFAULT_LASTMOD = "Sun, 01 Jan 2000 00:00:01 GMT"


def add_default_response():
    responses.add(
        "GET",
        DEFAULT_RESPONSE_URL,
        headers={"Last-Modified": DEFAULT_LASTMOD},
        json={},
        match_querystring=None,
    )


@pytest.fixture
def default_response():
    add_default_response()


def test_default_filename_from_uri(default_response):
    cd = CacheDownloader("downloads").bind(DEFAULT_RESPONSE_URL)
    assert cd._filename == url_to_cache_filename(DEFAULT_RESPONSE_URL)


@pytest.mark.parametrize(
    "sysname, fakeenv, expect_value",
    [
        ("Windows", {}, None),
        (
            "Windows",
            {"LOCALAPPDATA": "localappdata", "APPDATA": "appdata"},
            "localappdata",
        ),
        ("Windows", {"LOCALAPPDATA": "localappdata"}, "localappdata"),
        ("Windows", {"APPDATA": "appdata"}, "appdata"),
        ("Darwin", {}, "<expanduser>"),
        ("Linux", {}, "<expanduser>"),
        ("Linux", {"XDG_CACHE_HOME": "xdg-cache"}, "xdg-cache"),
    ],
)
def test_default_cache_dir(
    patch_cache_dir, monkeypatch, default_response, sysname, fakeenv, expect_value
):
    # undo the patch which typically overrides resolution of the cache dir
    patch_cache_dir.undo()

    for var in ["LOCALAPPDATA", "APPDATA", "XDG_CACHE_HOME"]:
        monkeypatch.delenv(var, raising=False)
    for k, v in fakeenv.items():
        monkeypatch.setenv(k, v)
    if expect_value is not None:
        expect_value = os.path.join(expect_value, "check_jsonschema", "downloads")

    def fakesystem():
        return sysname

    expanduser_path = None

    def fake_expanduser(path):
        nonlocal expanduser_path
        expanduser_path = path
        return "<expanduser>"

    monkeypatch.setattr(platform, "system", fakesystem)
    monkeypatch.setattr(os.path, "expanduser", fake_expanduser)

    cd = CacheDownloader("downloads")
    assert cd._cache_dir == expect_value

    if sysname == "Darwin":
        assert expanduser_path == "~/Library/Caches"
    elif sysname == "Linux":
        assert expanduser_path == "~/.cache"
    else:
        assert expanduser_path is None


def test_cache_hit_by_mtime(monkeypatch, default_response):
    monkeypatch.setattr(os.path, "exists", lambda x: True)

    # local mtime = NOW, cache hit
    monkeypatch.setattr(os.path, "getmtime", lambda x: time.time())
    assert _cache_hit(
        "/tmp/schema1.json",
        requests.get(DEFAULT_RESPONSE_URL, stream=True),
    )

    # local mtime = 0, cache miss
    monkeypatch.setattr(os.path, "getmtime", lambda x: 0)
    assert (
        _cache_hit(
            "/tmp/schema1.json",
            requests.get(DEFAULT_RESPONSE_URL, stream=True),
        )
        is False
    )


def test_cachedownloader_cached_file(tmp_path, monkeypatch, default_response):
    # create a file
    f = tmp_path / "foo.json"
    f.write_text("{}")

    # set the cache_dir to the tmp dir (so that cache_dir will always be set)
    cd = CacheDownloader(tmp_path).bind(str(f), filename="foo.json")
    # patch the downloader to skip any download "work"
    monkeypatch.setattr(
        cd._downloader, "_download", lambda file_uri, filename, response_ok: str(f)
    )

    with cd.open() as fp:
        assert fp.read() == b"{}"


@pytest.mark.parametrize("disable_cache", (True, False))
def test_cachedownloader_on_success(
    get_download_cache_loc, disable_cache, default_response
):
    f = get_download_cache_loc(DEFAULT_RESPONSE_URL)
    cd = CacheDownloader("downloads", disable_cache=disable_cache).bind(
        DEFAULT_RESPONSE_URL
    )

    with cd.open() as fp:
        assert fp.read() == b"{}"
    if disable_cache:
        assert not f.exists()
    else:
        assert f.exists()


def test_cachedownloader_using_alternate_target_dir(
    cache_dir, default_response, url2cachepath
):
    cache_dir = cache_dir / "check_jsonschema" / "otherdir"
    f = url2cachepath(cache_dir, DEFAULT_RESPONSE_URL)
    cd = CacheDownloader("otherdir").bind(DEFAULT_RESPONSE_URL)
    with cd.open() as fp:
        assert fp.read() == b"{}"
    assert f.exists()


@pytest.mark.parametrize("disable_cache", (True, False))
@pytest.mark.parametrize("failures", (1, 2, requests.ConnectionError))
def test_cachedownloader_succeeds_after_few_errors(
    get_download_cache_loc, disable_cache, failures
):
    if isinstance(failures, int):
        for _i in range(failures):
            responses.add(
                "GET",
                DEFAULT_RESPONSE_URL,
                status=500,
                match_querystring=None,
            )
    else:
        responses.add(
            "GET",
            DEFAULT_RESPONSE_URL,
            body=failures(),
            match_querystring=None,
        )
    add_default_response()
    f = get_download_cache_loc(DEFAULT_RESPONSE_URL)
    cd = CacheDownloader("downloads", disable_cache=disable_cache).bind(
        DEFAULT_RESPONSE_URL
    )

    with cd.open() as fp:
        assert fp.read() == b"{}"
    if disable_cache:
        assert not f.exists()
    else:
        assert f.exists()


@pytest.mark.parametrize("disable_cache", (True, False))
@pytest.mark.parametrize("connection_error", (True, False))
def test_cachedownloader_fails_after_many_errors(
    get_download_cache_loc, disable_cache, connection_error
):
    for _i in range(10):
        if connection_error:
            responses.add(
                "GET",
                DEFAULT_RESPONSE_URL,
                body=requests.ConnectionError(),
                match_querystring=None,
            )
        else:
            responses.add(
                "GET",
                DEFAULT_RESPONSE_URL,
                status=500,
                match_querystring=None,
            )
    add_default_response()  # never reached, the 11th response
    f = get_download_cache_loc(DEFAULT_RESPONSE_URL)
    cd = CacheDownloader("downloads", disable_cache=disable_cache).bind(
        DEFAULT_RESPONSE_URL
    )
    with pytest.raises(FailedDownloadError):
        with cd.open():
            pass
    assert not f.exists()


@pytest.mark.parametrize("disable_cache", (True, False))
def test_cachedownloader_retries_on_bad_data(get_download_cache_loc, disable_cache):
    responses.add(
        "GET",
        DEFAULT_RESPONSE_URL,
        status=200,
        body="{",
        match_querystring=None,
    )
    add_default_response()
    f = get_download_cache_loc(DEFAULT_RESPONSE_URL)
    cd = CacheDownloader(
        "downloads",
        disable_cache=disable_cache,
    ).bind(
        DEFAULT_RESPONSE_URL,
        validation_callback=json.loads,
    )

    with cd.open() as fp:
        assert fp.read() == b"{}"

    if disable_cache:
        assert not f.exists()
    else:
        assert f.exists()


@pytest.mark.parametrize("file_exists", (True, False))
@pytest.mark.parametrize(
    "failure_mode", ("header_missing", "header_malformed", "time_overflow")
)
def test_cachedownloader_handles_bad_lastmod_header(
    monkeypatch,
    get_download_cache_loc,
    inject_cached_download,
    file_exists,
    failure_mode,
):
    if failure_mode == "header_missing":
        responses.add(
            "GET", DEFAULT_RESPONSE_URL, headers={}, json={}, match_querystring=None
        )
    elif failure_mode == "header_malformed":
        responses.add(
            "GET",
            DEFAULT_RESPONSE_URL,
            headers={"Last-Modified": "Jan 2000 00:00:01"},
            json={},
            match_querystring=None,
        )
    elif failure_mode == "time_overflow":
        add_default_response()

        def fake_timegm(*args):
            raise OverflowError("uh-oh")

        monkeypatch.setattr("calendar.timegm", fake_timegm)
    else:
        raise NotImplementedError

    original_file_contents = b'{"foo": "bar"}'
    file_path = get_download_cache_loc(DEFAULT_RESPONSE_URL)

    assert not file_path.exists()
    if file_exists:
        inject_cached_download(DEFAULT_RESPONSE_URL, original_file_contents)

    cd = CacheDownloader("downloads").bind(DEFAULT_RESPONSE_URL)

    # if the file already existed, it will not be overwritten by the cachedownloader
    # so the returned value for both the downloader and a direct file read should be the
    # original contents
    if file_exists:
        with cd.open() as fp:
            assert fp.read() == original_file_contents
        assert file_path.read_bytes() == original_file_contents
    # otherwise, the file will have been created with new content
    # both reads will show that new content
    else:
        with cd.open() as fp:
            assert fp.read() == b"{}"
        assert file_path.read_bytes() == b"{}"

    # at the end, the file always exists on disk
    assert file_path.exists()


def test_cachedownloader_validation_is_not_invoked_on_hit(
    monkeypatch, default_response, inject_cached_download
):
    """
    Regression test for https://github.com/python-jsonschema/check-jsonschema/issues/453

    This was a bug in which the validation callback was invoked eagerly, even on a cache
    hit. As a result, cache hits did not demonstrate their expected performance gain.
    """
    # 1: construct some perfectly good data (it doesn't really matter what it is)
    #    <<default_response fixture>>
    # 2: put equivalent data on disk
    inject_cached_download(DEFAULT_RESPONSE_URL, "{}")

    # 3: construct a validator which marks that it ran in a variable
    validator_ran = False

    def dummy_validate_bytes(data):
        nonlocal validator_ran
        validator_ran = True

    # construct a downloader pointed at the schema and file, expecting a cache hit
    # and use the above validation method
    cd = CacheDownloader("downloads").bind(
        DEFAULT_RESPONSE_URL,
        validation_callback=dummy_validate_bytes,
    )

    # read data from the downloader
    with cd.open() as fp:
        assert fp.read() == b"{}"
    # assert that the validator was not run
    assert validator_ran is False


@pytest.mark.skipif(
    platform.system() == "Windows",
    reason="time.tzset() is not available on Windows",
)
def test_lastmod_from_header_uses_gmtime(request, monkeypatch, default_response):
    """
    Regression test for https://github.com/python-jsonschema/check-jsonschema/pull/565

    The time was converted in local time, when UTC/GMT was desired.
    """

    def final_tzset():
        time.tzset()

    request.addfinalizer(final_tzset)

    response = requests.get(DEFAULT_RESPONSE_URL, stream=True)

    with monkeypatch.context() as m:
        m.setenv("TZ", "GMT0")
        time.tzset()
        gmt_parsed_time = _lastmod_from_response(response)

    with monkeypatch.context() as m:
        m.setenv("TZ", "EST5")
        time.tzset()
        est_parsed_time = _lastmod_from_response(response)

    with monkeypatch.context() as m:
        m.setenv("TZ", "UTC0")
        time.tzset()
        utc_parsed_time = _lastmod_from_response(response)

    # assert that they all match
    assert gmt_parsed_time == utc_parsed_time
    assert gmt_parsed_time == est_parsed_time



#!/usr/bin/env python
import re
import sys


def get_old_version():
    with open("pyproject.toml") as fp:
        content = fp.read()
    match = re.search(r'^version = "(\d+\.\d+\.\d+)"$', content, flags=re.MULTILINE)
    assert match
    return match.group(1)


def replace_version(filename, formatstr, old_version, new_version):
    print(f"updating {filename}")
    with open(filename) as fp:
        content = fp.read()
    old_str = formatstr.format(old_version)
    new_str = formatstr.format(new_version)
    content = content.replace(old_str, new_str)
    with open(filename, "w") as fp:
        fp.write(content)


def update_changelog(new_version):
    print("updating CHANGELOG.rst")
    with open("CHANGELOG.rst") as fp:
        content = fp.read()

    vendor_marker = ".. vendor-insert-here"

    content = re.sub(
        r"""
Unreleased
----------
(\s*\n)+""" + re.escape(vendor_marker),
        f"""
Unreleased
----------

{vendor_marker}

{new_version}
{"-" * len(new_version)}""",
        content,
    )
    with open("CHANGELOG.rst", "w") as fp:
        fp.write(content)


def parse_version(s):
    vals = s.split(".")
    assert len(vals) == 3
    return tuple(int(x) for x in vals)


def comparse_versions(old_version, new_version):
    assert parse_version(new_version) > parse_version(old_version)


def main():
    if len(sys.argv) != 2:
        sys.exit(2)

    new_version = sys.argv[1]
    old_version = get_old_version()
    print(f"old = {old_version}, new = {new_version}")
    comparse_versions(old_version, new_version)

    replace_version("pyproject.toml", 'version = "{}"', old_version, new_version)
    replace_version("README.md", "rev: {}", old_version, new_version)
    replace_version("docs/precommit_usage.rst", "rev: {}", old_version, new_version)
    replace_version("docs/optional_parsers.rst", "rev: {}", old_version, new_version)
    update_changelog(new_version)


if __name__ == "__main__":
    main()
